@@ -74,6 +74,7 @@ import com.google.common.collect.ImmutableList;
 import com.openexchange.config.cascade.ConfigView;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.config.cascade.ConfigViews;
+import com.openexchange.context.ContextService;
 import com.openexchange.database.DatabaseService;
 import com.openexchange.exception.OXException;
 import com.openexchange.gdpr.dataexport.DataExport;
@@ -95,6 +96,7 @@ import com.openexchange.gdpr.dataexport.DataExportWorkItem;
 import com.openexchange.gdpr.dataexport.DayOfWeekTimeRanges;
 import com.openexchange.gdpr.dataexport.DefaultDataExport;
 import com.openexchange.gdpr.dataexport.DefaultDataExportResultFile;
+import com.openexchange.gdpr.dataexport.DiagnosticsReportOptions;
 import com.openexchange.gdpr.dataexport.FileLocation;
 import com.openexchange.gdpr.dataexport.FileLocations;
 import com.openexchange.gdpr.dataexport.HostInfo;
@@ -106,11 +108,15 @@ import com.openexchange.gdpr.dataexport.impl.notification.Reason;
 import com.openexchange.java.ISO8601Utils;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
+import com.openexchange.threadpool.AbstractTask;
 import com.openexchange.threadpool.ThreadPoolService;
+import com.openexchange.threadpool.ThreadPools;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
 import com.openexchange.user.User;
 import com.openexchange.user.UserService;
+import gnu.trove.map.TIntObjectMap;
+import gnu.trove.map.hash.TIntObjectHashMap;
 
 
 /**
@@ -264,8 +270,94 @@ public class DataExportServiceImpl implements DataExportService {
     }
 
     @Override
-    public List<DataExportTask> getDataExportTasks() throws OXException {
-        return storageService.getDataExportTasks();
+    public List<DataExportTask> getDataExportTasks(boolean checkValidity) throws OXException {
+        if (checkValidity == false) {
+            return storageService.getDataExportTasks();
+        }
+
+        List<DataExportTask> dataExportTasks = storageService.getDataExportTasks();
+        if (dataExportTasks.isEmpty()) {
+            return dataExportTasks;
+        }
+
+        ContextService contextService = services.getOptionalService(ContextService.class);
+        if (contextService != null) {
+            UserService userService = services.getOptionalService(UserService.class);
+            if (userService != null) {
+                dataExportTasks = checkTasksValidity(dataExportTasks, userService, contextService);
+            }
+        }
+
+        return dataExportTasks;
+    }
+
+    private List<DataExportTask> checkTasksValidity(List<DataExportTask> dataExportTasks, UserService userService, ContextService contextService) {
+        List<DataExportTask> tasksToDelete = null;
+
+        TIntObjectMap<Boolean> visitedContexts = new TIntObjectHashMap<>();
+        for (Iterator<DataExportTask> it = dataExportTasks.iterator(); it.hasNext();) {
+            DataExportTask dataExportTask = it.next();
+
+            int contextId = dataExportTask.getContextId();
+            Boolean exists = visitedContexts.get(contextId);
+            if (exists == null) {
+                exists = Boolean.valueOf(contextExists(contextId, contextService));
+                visitedContexts.put(contextId, exists);
+            }
+
+            if (exists.booleanValue()) {
+                if (userExists(dataExportTask.getUserId(), contextId, userService) == false) {
+                    // Remove task
+                    it.remove();
+                    if (tasksToDelete == null) {
+                        tasksToDelete = new ArrayList<>();
+                    }
+                    tasksToDelete.add(dataExportTask);
+                }
+            } else {
+                // Remove task
+                it.remove();
+                if (tasksToDelete == null) {
+                    tasksToDelete = new ArrayList<>();
+                }
+                tasksToDelete.add(dataExportTask);
+            }
+        }
+
+        if (tasksToDelete != null) {
+            List<DataExportTask> tasks = tasksToDelete;
+            DataExportStorageService storageService = this.storageService;
+            AbstractTask<Void> deleteTask = new AbstractTask<Void>() {
+
+                @Override
+                public Void call() throws Exception {
+                    for (DataExportTask task : tasks) {
+                        storageService.deleteDataExportTask(task.getId());
+                        LOG.info("Deleted data export task {} of user {} in context {} since associated user does not exist.", DataExportUtility.stringFor(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                    }
+                    return null;
+                }
+            };
+            ThreadPools.submitElseExecute(deleteTask);
+        }
+
+        return dataExportTasks;
+    }
+
+    private boolean contextExists(int contextId, ContextService contextService) {
+        try {
+            return contextService.exists(contextId);
+        } catch (Exception e) {
+            return true;
+        }
+    }
+
+    private boolean userExists(int userId, int contextId, UserService userService) {
+        try {
+            return userService.exists(userId, contextId);
+        } catch (Exception e) {
+            return true;
+        }
     }
 
     @Override
@@ -325,10 +417,31 @@ public class DataExportServiceImpl implements DataExportService {
         task.setWorkItems(workItems);
         task.setArguments(args);
 
-        boolean created = storageService.createIfAbsent(task, userId, contextId);
-        if (created) {
-            LOG.info("Submitted data export task {} for user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(contextId));
-            return Optional.of(taskId);
+        boolean keepgoing = true;
+        while (keepgoing) {
+            boolean created = storageService.createIfAbsent(task, userId, contextId);
+            if (created) {
+                LOG.info("Submitted data export task {} for user {} in context {}", stringFor(task.getId()), I(task.getUserId()), I(contextId));
+                return Optional.of(taskId);
+            }
+
+            // Drop aborted ones
+            keepgoing = false;
+            try {
+                Optional<DataExportTask> optExisting = storageService.getDataExportTask(userId, contextId);
+                if (optExisting.isPresent()) {
+                    DataExportTask existing = optExisting.get();
+                    if (DataExportStatus.ABORTED == existing.getStatus()) {
+                        boolean deleted = storageService.deleteDataExportTask(userId, contextId);
+                        if (deleted) {
+                            keepgoing = true;
+                            LOG.info("Deleted aborted data export task {} (incl. all resources and artifacts) of user {} in context {} in order to submit a new task", stringFor(existing.getId()), I(task.getUserId()), I(contextId));
+                        }
+                    }
+                }
+            } catch (Exception e) {
+                LOG.error("Failed checking for existent aborted data export task of user {} in context {}", I(task.getUserId()), I(contextId));
+            }
         }
         return Optional.empty();
     }
@@ -732,7 +845,7 @@ public class DataExportServiceImpl implements DataExportService {
             throw new IllegalArgumentException("Start delay must not be less than 0 (zero): " + startDelay);
         }
         if (stopDelay < startDelay) {
-            throw new IllegalArgumentException("Stop delay (" + startDelay + ") must be greater than start delay (" + stopDelay + ")");
+            throw new IllegalArgumentException("Stop delay (" + stopDelay + ") must be greater than start delay (" + startDelay + ")");
         }
         final long endTimeMillis = currentTimeMillis + stopDelay;
         Runnable startTask = new Runnable() {
@@ -810,10 +923,9 @@ public class DataExportServiceImpl implements DataExportService {
         DatabaseService databaseService = services.getServiceSafe(DatabaseService.class);
 
         DataExportLock lock = DataExportLock.getInstance();
-        boolean acquired = false;
+        DataExportLock.LockAcquisition acquisition = lock.acquireCleanUpTaskLock(false, databaseService);
         try {
-            acquired = lock.acquireCleanUpTaskLock(false, databaseService);
-            if (acquired == false) {
+            if (acquisition.isNotAcquired()) {
                 // Failed to acquire clean-up lock
                 LOG.info("Failed to acquire clean-up lock for data export files. Skipping job execution run...");
                 return;
@@ -848,8 +960,11 @@ public class DataExportServiceImpl implements DataExportService {
                 long nanosToWait = TimeUnit.NANOSECONDS.convert((count++ * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
                 LockSupport.parkNanos(nanosToWait);
 
-                // Check if a diagnostics report is supposed to be compiled for task-associated user
-                boolean addDiagnosticsReport = isAddDiagnosticsReport(job);
+                // Diagnostics report
+                DiagnosticsReportOptions reportOptions = DiagnosticsReportOptions.builder()
+                    .withAddDiagnosticsReport(isAddDiagnosticsReport(job))
+                    .withConsiderPermissionDeniedErrors(isConsiderPermissionDeniedErrors(job))
+                    .build();
 
                 // Some variables for clean-up
                 DataExportTaskExecution execution = null;
@@ -857,7 +972,7 @@ public class DataExportServiceImpl implements DataExportService {
                 boolean openedForProcessing = false;
                 try {
                     // Initialize execution & submit it to thread pool
-                    execution = new DataExportTaskExecution(job, addDiagnosticsReport, config, storageService, providerRegistry, services);
+                    execution = new DataExportTaskExecution(job, reportOptions, config, storageService, providerRegistry, services);
                     future = threadPool.submit(execution);
 
                     // Store execution in map. Then open it for being processed while registering a clean-up task that ensures execution is removed from map when finished
@@ -878,12 +993,10 @@ public class DataExportServiceImpl implements DataExportService {
                 }
             }
         } finally {
-            if (acquired) {
-                try {
-                    lock.releaseCleanUpTaskLock(false, databaseService);
-                } catch (Exception e) {
-                    LOG.warn("Failed to release clean-up lock for data export files", e);
-                }
+            try {
+                lock.releaseCleanUpTaskLock(acquisition, databaseService);
+            } catch (Exception e) {
+                LOG.warn("Failed to release clean-up lock for data export files", e);
             }
         }
     }
@@ -899,6 +1012,17 @@ public class DataExportServiceImpl implements DataExportService {
         return addDiagnosticsReport;
     }
 
+    private boolean isConsiderPermissionDeniedErrors(DataExportJob job) throws OXException {
+        boolean addDiagnosticsReport = false;
+        ConfigViewFactory viewFactory = services.getOptionalService(ConfigViewFactory.class);
+        if (viewFactory != null) {
+            DataExportTask task = job.getDataExportTask();
+            ConfigView view = viewFactory.getView(task.getUserId(), job.getDataExportTask().getContextId());
+            addDiagnosticsReport = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.gdpr.dataexport.considerPermissionDeniedErrors", addDiagnosticsReport, view);
+        }
+        return addDiagnosticsReport;
+    }
+
     synchronized void stopProcessingTasks(boolean cancelStartStopTimerTasks) {
         if (cancelStartStopTimerTasks) {
             cancelTimerTasks(periodicStartTask, false, false);
