@@ -91,7 +91,6 @@ import com.openexchange.gdpr.dataexport.DataExportWorkItem;
 import com.openexchange.gdpr.dataexport.impl.DataExportLock;
 import com.openexchange.gdpr.dataexport.impl.DataExportUtility;
 import com.openexchange.java.ConvertUtils;
-import com.openexchange.reseller.ResellerService;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 
@@ -192,13 +191,13 @@ public class DataExportCleanUpTask implements Runnable {
         }
 
         DataExportLock lock = DataExportLock.getInstance();
-        boolean acquired = false;
         Thread currentThread = Thread.currentThread();
         String prevName = currentThread.getName();
         currentThread.setName("DataExportCleanUpTask");
+        DataExportLock.LockAcquisition acquisition = null;
         try {
-            acquired = lock.acquireCleanUpTaskLock(true, databaseService);
-            if (acquired == false) {
+            acquisition = lock.acquireCleanUpTaskLock(true, databaseService);
+            if (acquisition.isNotAcquired()) {
                 // Failed to acquire clean-up lock
                 LOG.info("Failed to acquire clean-up lock for data export files since another process is currently running. Aborting clean-up run...");
                 return;
@@ -221,9 +220,9 @@ public class DataExportCleanUpTask implements Runnable {
             ExceptionUtils.handleThrowable(t);
             LOG.warn("Failed clean-up run for data exports", t);
         } finally {
-            if (acquired) {
+            if (acquisition != null) {
                 try {
-                    lock.releaseCleanUpTaskLock(true, databaseService);
+                    lock.releaseCleanUpTaskLock(acquisition, databaseService);
                 } catch (Exception e) {
                     LOG.warn("Failed to release clean-up lock for data export files", e);
                 }
@@ -263,7 +262,7 @@ public class DataExportCleanUpTask implements Runnable {
                 storageService.markWorkItemPending(task.getId(), item.getModuleId(), task.getUserId(), task.getContextId());
                 LOG.debug("Marked orphaned work item {} of data export task from user {} in context {} as pending (missing task result files file storage) to enforce re-execution", item.getModuleId(), I(task.getUserId()), I(task.getContextId()));
             }
-            storageService.deleteResultFiles(task.getId(), task.getContextId());
+            storageService.deleteResultFiles(task.getId(), task.getUserId(), task.getContextId());
         }
     }
 
@@ -305,7 +304,7 @@ public class DataExportCleanUpTask implements Runnable {
             }
         }
 
-        for (DataExportTask task : dataExportService.getDataExportTasks()) {
+        for (DataExportTask task : dataExportService.getDataExportTasks(true)) {
             if (task.getResultFiles() != null) {
                 for (DataExportResultFile resultFile : task.getResultFiles()) {
                     if (resultFile.getFileStorageLocation() != null) {
@@ -338,7 +337,7 @@ public class DataExportCleanUpTask implements Runnable {
     }
 
     private List<Pair<DataExportTask, DataExportWorkItem>> getOrphanedWorkItems() throws OXException {
-        List<DataExportTask> tasks = dataExportService.getDataExportTasks();
+        List<DataExportTask> tasks = dataExportService.getDataExportTasks(false);
         List<Pair<DataExportTask, DataExportWorkItem>> retval = null;
         for (DataExportTask task : tasks) {
             SortedSet<String> fileList = DataExportUtility.getFileStorageFor(task).getFileList();
@@ -355,7 +354,7 @@ public class DataExportCleanUpTask implements Runnable {
     }
 
     private List<Pair<DataExportTask, DataExportResultFile>> getOrphanedResultFiles() throws OXException {
-        List<DataExportTask> tasks = dataExportService.getDataExportTasks();
+        List<DataExportTask> tasks = dataExportService.getDataExportTasks(false);
         List<Pair<DataExportTask, DataExportResultFile>> retval = null;
         for (DataExportTask task : tasks) {
             if (task.getResultFiles() != null && !task.getResultFiles().isEmpty()) {
@@ -392,22 +391,6 @@ public class DataExportCleanUpTask implements Runnable {
             LOG.warn("Failed to retrieve possible identifiers of data export file storage from context-sets scope", e);
         }
 
-        // Check for possible configured file storage identifiers for reseller scope
-        try {
-            ResellerService resellerService = services.getOptionalService(ResellerService.class);
-            if (resellerService != null) {
-                Set<Integer> resellerFileStoreIds = determineResellerFileStoreIds(databaseService);
-                if (resellerFileStoreIds.isEmpty() == false) {
-                    if (fileStoreIds == null) {
-                        fileStoreIds = new HashSet<>();
-                    }
-                    fileStoreIds.addAll(resellerFileStoreIds);
-                }
-            }
-        } catch (Exception e) {
-            LOG.warn("Failed to retrieve possible identifiers of data export file storage from reseller scope", e);
-        }
-
         // Check for possible configured file storage identifiers for server scope
         try {
             ConfigurationService configurationService = services.getOptionalService(ConfigurationService.class);
@@ -487,39 +470,6 @@ public class DataExportCleanUpTask implements Runnable {
         return fileStoreIds == null ? Collections.emptySet() : fileStoreIds;
     }
 
-    private Set<Integer> determineResellerFileStoreIds(DatabaseService databaseService) throws OXException {
-        PreparedStatement stmt = null;
-        ResultSet rs = null;
-        Connection con = databaseService.getReadOnly();
-        try {
-            if (Databases.tableExists(con, "subadmin_config_properties") == false) {
-                return Collections.emptySet();
-            }
-
-            stmt = con.prepareStatement("SELECT propertyValue FROM subadmin_config_properties WHERE propertyKey = ?");
-            stmt.setString(1, "com.openexchange.gdpr.dataexport.fileStorageId");
-            rs = stmt.executeQuery();
-            if (rs.next() == false) {
-                return Collections.emptySet();
-            }
-
-            Set<Integer> fileStoreIds = new HashSet<>();
-            do {
-                try {
-                    fileStoreIds.add(Integer.valueOf(rs.getString(1)));
-                } catch (NumberFormatException e) {
-                    // Ignore
-                }
-            } while (rs.next());
-            return fileStoreIds;
-        } catch (SQLException e) {
-            throw DataExportExceptionCode.SQL_ERROR.create(e, e.getMessage());
-        } finally {
-            Databases.closeSQLStuff(rs, stmt);
-            databaseService.backReadOnly(con);
-        }
-    }
-
     private Set<Integer> determineFileStoreIdsFor(int representativeContextId, DatabaseService databaseService) throws OXException {
         PreparedStatement stmt = null;
         ResultSet rs = null;
