@@ -186,7 +186,7 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
     @Override
     public List<Map<String, Object>> getDataExportTasks() throws RemoteException {
         try {
-            List<DataExportTask> tasks = service.getDataExportTasks();
+            List<DataExportTask> tasks = service.getDataExportTasks(false);
 
             List<Map<String, Object>> retval = new ArrayList<>(tasks.size());
             for (DataExportTask task : tasks) {
@@ -262,7 +262,7 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
                 for (DataExportWorkItem item : task.getWorkItems()) {
                     storageService.markWorkItemPending(task.getId(), item.getModuleId(), task.getUserId(), task.getContextId());
                 }
-                storageService.deleteResultFiles(task.getId(), task.getContextId());
+                storageService.deleteResultFiles(task.getId(), task.getUserId(), task.getContextId());
                 fixed++;
             }
         } catch (OXException e) {
@@ -300,7 +300,7 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
                 }
             }
 
-            for (DataExportTask task : service.getDataExportTasks()) {
+            for (DataExportTask task : service.getDataExportTasks(false)) {
                 if (task.getResultFiles() != null) {
                     for (DataExportResultFile resultFile : task.getResultFiles()) {
                         if (resultFile.getFileStorageLocation() != null) {
@@ -323,7 +323,7 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
 
     private List<Pair<DataExportTask, DataExportWorkItem>> getOrphanedWorkItems_() throws RemoteException {
         try {
-            List<DataExportTask> tasks = service.getDataExportTasks();
+            List<DataExportTask> tasks = service.getDataExportTasks(false);
             List<Pair<DataExportTask, DataExportWorkItem>> retval = null;
             for (DataExportTask task : tasks) {
                 SortedSet<String> fileList = DataExportUtility.getFileStorageFor(task).getFileList();
@@ -345,7 +345,7 @@ public class DataExportRMIServiceImpl implements DataExportRMIService {
 
     private List<Pair<DataExportTask, DataExportResultFile>> getOrphanedResultFiles_() throws RemoteException {
         try {
-            List<DataExportTask> tasks = service.getDataExportTasks();
+            List<DataExportTask> tasks = service.getDataExportTasks(false);
             List<Pair<DataExportTask, DataExportResultFile>> retval = null;
             for (DataExportTask task : tasks) {
                 if (task.getResultFiles() != null && !task.getResultFiles().isEmpty()) {
